@@ -1,17 +1,21 @@
-#! /usr/bin/python
+#! /usr/bin/python3
 # Written by Dan Mandle http://dan.mandle.me September 2012
 # http://www.danmandle.com/blog/getting-gpsd-to-work-with-python/
 # License: GPL 2.0
 
 # This code runs compatibly under Python 2 and 3.x for x >= 2.
 # Preserve this property!
+# Codacy D203 and D211 conflict, I choose D203
+# Codacy D212 and D213 conflict, I choose D212
+
+"""gpsd data dump example."""
+
 from __future__ import absolute_import, print_function, division
 
+import gps
 import os
-from gps import *
-from time import *
-import time
 import threading
+import time
 
 gpsd = None          # setting the global variable
 
@@ -19,14 +23,18 @@ os.system('clear')   # clear the terminal (optional)
 
 
 class GpsPoller(threading.Thread):
+    """Class to poll gpsd"""
+
     def __init__(self):
+        """Initializer for GpsPoller."""
         threading.Thread.__init__(self)
         global gpsd                    # bring it in scope
-        gpsd = gps(mode=WATCH_ENABLE)  # starting the stream of info
+        gpsd = gps.gps(mode=gps.WATCH_ENABLE)  # starting the stream of info
         self.current_value = None
         self.running = True            # setting the thread running to true
 
     def run(self):
+        """Run."""
         global gpsd
         while gpsp.running:
             # this will continue to loop and grab EACH set of
@@ -51,7 +59,7 @@ if __name__ == '__main__':
             print('latitude    ', gpsd.fix.latitude)
             print('longitude   ', gpsd.fix.longitude)
             print('time utc    ', gpsd.utc, ' + ', gpsd.fix.time)
-            print('altitude (m)', gpsd.fix.altitude)
+            print('altHAE (m)  ', gpsd.fix.altHAE)
             print('eps         ', gpsd.fix.eps)
             print('epx         ', gpsd.fix.epx)
             print('epv         ', gpsd.fix.epv)
