@@ -1,95 +1,103 @@
-#!/usr/bin/env python
-#
-# This tool is intended to automate away the drudgery in bring up support
-# for a new AIS message type.  It parses the tabular description of a message
-# and generates various useful code snippets from that.  It can also be used to
-# correct offsets in the tables themselves.
-#
-# Requires the AIVDM.txt file on standard input. Takes a single argument,
-# which must match a string in a //: Type comment.  Things you can generate:
-#
-# * -t: A corrected version of the table.  It will redo all the offsets to be
-#   in conformance with the bit widths. (The other options rely only on the
-#   bit widths). If the old and new tables are different, an error message
-#   describing the corrections will be emitted to standard error.
-#
-# * -s: A structure definition capturing the message info, with member
-#   names extracted from the table and types computed from it.
-#
-# * -c: Bit-extraction code for the AIVDM driver.  Grind out the right sequence
-#   of UBITS, SBITS, and UCHARS macros, and assignments to structure members,
-#   guaranteed correct if the table offsets and widths are.
-#
-# * -d: Code to dump the contents of the unpacked message structure as JSON. If
-#   the structure has float members, you'll get an if/then/else  guarded by
-#   the scaled flag.
-#
-# * -r: A Python initializer stanza for jsongen.py, which is in turn used to
-#   generate the specification structure for a JSON parse that reads JSON
-#   into an instance of the message structure.
-#
-# * -a: Generate all of -s, -d, -c, and -r, and -t, not to stdout but to
-#   files named with 'tablegen' as a distinguishing part of the stem.
-#   The stem name can be overridden with the -o option.
-#
-# This generates almost all the code required to support a new message type.
-# It's not quite "Look, ma, no handhacking!" You'll need to add default
-# values to the Python stanza. If the structure definition contains character
-# arrays, you'll have to fill in the dimensions by hand.  You'll need to add
-# a bit of glue to ais_json.c so that json_ais_read() actually calls the parser
-# handing it the specification structure as a control argument.
-#
-# The -a, -c, -s, -d, and -r modes all take an argument, which should be a
-# structure reference prefix to be prepended (before a dot) to each fieldname.
-# Usually you'll need this to look something like "ais->typeN", but it could be
-# "ais->typeN.FOO" if the generated code has to operate on a union member
-# inside a type 6 or 8, or something similar.
-#
-# The -S and -E options allow you to generate code only for a specified span
-# of fields in the table.  This may be useful for dealing with groups of
-# messages that have a common head section.
-#
-# This code interprets magic comments in the input
-#
-# //: Type
-#    The token following "Type" is the name of the table
-# //: xxxx vocabulary
-#    A subtable describing a controlled vocabulary for field xxxx in the
-#    preceding table.
-#
-# TO-DO: generate code for ais.py.
-#
+#!/usr/bin/env python3
 # This code runs compatibly under Python 2 and 3.x for x >= 2.
 # Preserve this property!
+
+"""Generate AIS code from text file.
+
+This tool is intended to automate away the drudgery in bring up support
+for a new AIS message type.  It parses the tabular description of a message
+and generates various useful code snippets from that.  It can also be used to
+correct offsets in the tables themselves.
+
+Requires the AIVDM.txt file on standard input. Takes a single argument,
+which must match a string in a //: Type comment.  Things you can generate:
+
+* -t: A corrected version of the table.  It will redo all the offsets to be
+  in conformance with the bit widths. (The other options rely only on the
+  bit widths). If the old and new tables are different, an error message
+  describing the corrections will be emitted to standard error.
+
+* -s: A structure definition capturing the message info, with member
+  names extracted from the table and types computed from it.
+
+* -c: Bit-extraction code for the AIVDM driver.  Grind out the right sequence
+  of UBITS, SBITS, and UCHARS macros, and assignments to structure members,
+  guaranteed correct if the table offsets and widths are.
+
+* -d: Code to dump the contents of the unpacked message structure as JSON. If
+  the structure has float members, you'll get an if/then/else  guarded by
+  the scaled flag.
+
+* -r: A Python initializer stanza for jsongen.py, which is in turn used to
+  generate the specification structure for a JSON parse that reads JSON
+  into an instance of the message structure.
+
+* -a: Generate all of -s, -d, -c, and -r, and -t, not to stdout but to
+  files named with 'tablegen' as a distinguishing part of the stem.
+  The stem name can be overridden with the -o option.
+
+This generates almost all the code required to support a new message type.
+It's not quite "Look, ma, no handhacking!" You'll need to add default
+values to the Python stanza. If the structure definition contains character
+arrays, you'll have to fill in the dimensions by hand.  You'll need to add
+a bit of glue to ais_json.c so that json_ais_read() actually calls the parser
+handing it the specification structure as a control argument.
+
+The -a, -c, -s, -d, and -r modes all take an argument, which should be a
+structure reference prefix to be prepended (before a dot) to each fieldname.
+Usually you'll need this to look something like "ais->typeN", but it could be
+"ais->typeN.FOO" if the generated code has to operate on a union member
+inside a type 6 or 8, or something similar.
+
+The -S and -E options allow you to generate code only for a specified span
+of fields in the table.  This may be useful for dealing with groups of
+messages that have a common head section.
+
+This code interprets magic comments in the input
+
+//: Type
+   The token following "Type" is the name of the table
+//: xxxx vocabulary
+   A subtable describing a controlled vocabulary for field xxxx in the
+   preceding table.
+
+TO-DO: generate code for ais.py.
+"""
+
 from __future__ import absolute_import, print_function, division
 
-import sys, getopt
+import getopt
+import sys
+
 
 def correct_table(wfp):
-    # Writes the corrected table.
+    """Writes the corrected table."""
     print("Total bits:", base, file=sys.stderr)
-    for (i, t) in enumerate(table):
-        if offsets[i].strip():
-            print("|" + offsets[i] + t[owidth+1:].rstrip(), file=wfp)
+    for (idx, t) in enumerate(table):
+        if offsets[idx].strip():
+            print("|" + offsets[idx] + t[owidth+1:].rstrip(), file=wfp)
         else:
             print(t.rstrip(), file=wfp)
 
+
 def make_driver_code(wfp):
-    # Writes calls to bit-extraction macros.
-    # Requires UBITS, SBITS, UCHARS to act as they do in the AIVDM driver.
-    # Also relies on bitlen to be the message bit length, and i to be
-    # available as abn index variable.
+    """Writes calls to bit-extraction macros.
+    Requires UBITS, SBITS, UCHARS to act as they do in the AIVDM driver.
+    Also relies on bitlen to be the message bit length, and i to be
+    available as abn index variable."""
     record = after is None
     arrayname = None
-    base = '\t'
+    lbase = '\t'
     step = " " * 4
-    indent = base
+    indent = lbase
+    last = 0
+
     for (i, t) in enumerate(table):
         if '|' in t:
-            fields = [s.strip() for s in t.split('|')]
-            width = fields[2]
-            name = fields[4]
-            ftype = fields[5]
+            filds = [s.strip() for s in t.split('|')]
+            width = filds[2]
+            name = filds[4]
+            ftype = filds[5]
             if after == name:
                 record = True
                 continue
@@ -99,8 +107,8 @@ def make_driver_code(wfp):
             if not record:
                 continue
             if ftype == 'x':
-                print("\t/* skip %s bit%s */" % (width, ["", "s"][width>'1']),
-                      file=wfp)
+                print("\t/* skip %s bit%s */" % (width,
+                      ["", "s"][width > '1']), file=wfp)
                 continue
             if ftype[0] == 'a':
                 arrayname = name
@@ -110,50 +118,59 @@ def make_driver_code(wfp):
                 if explicit:
                     lengthfield = last
                     print(indent + "for (i = 0; i < %s; i++) {" % lengthfield,
-                          file=wfp) 
+                          file=wfp)
                 else:
                     lengthfield = "n" + arrayname
-                    print(indent + "for (i = 0; ARRAY_BASE + (ELEMENT_SIZE*i) < bitlen; i++) {", file=wfp) 
+                    print(indent + "for (i = 0; ARRAY_BASE + "
+                          "(ELEMENT_SIZE*i) < bitlen; i++) {", file=wfp)
                 indent += step
                 print(indent + "int a = ARRAY_BASE + (ELEMENT_SIZE*i);",
-                      file=wfp) 
+                      file=wfp)
                 continue
             offset = offsets[i].split('-')[0]
             if arrayname:
                 target = "%s.%s[i].%s" % (structnme, arrayname, name)
-                offset = "a + " + offset 
+                offset = "a + " + offset
             else:
                 target = "%s.%s" % (structname, name)
             if ftype[0].lower() in ('u', 'i', 'e'):
-                print(indent + "%s\t= %sBITS(%s, %s);" % \
-                      (target, {'u':'U', 'e':'U', 'i':'S'}[ftype[0].lower()], offset, width), file=wfp)
+                print(indent + "%s\t= %sBITS(%s, %s);" %
+                      (target,
+                       {'u': 'U', 'e': 'U', 'i': 'S'}[ftype[0].lower()],
+                       offset, width), file=wfp)
             elif ftype == 't':
                 print(indent + "UCHARS(%s, %s);" % (offset, target), file=wfp)
             elif ftype == 'b':
                 print(indent + "%s\t= (bool)UBITS(%s, 1);" % (target, offset),
                       file=wfp)
             else:
-                print(indent + "/* %s bits of type %s */" % (width,ftype),
-                      file=wfp)
+                print(indent + "/* %s bits of type %s */" %
+                      (width, ftype), file=wfp)
             last = name
     if arrayname:
-        indent = base
+        indent = lbase
         print(indent + "}", file=wfp)
         if not explicit:
-            print(indent + "%s.%s = ind;" % (structname, lengthfield), file=wfp)
-        print("#undef ARRAY_BASE", file=wfp) 
-        print("#undef ELEMENT_SIZE", file=wfp) 
+            print(indent + "%s.%s = ind;" %
+                  (structname, lengthfield), file=wfp)
+        print("#undef ARRAY_BASE", file=wfp)
+        print("#undef ELEMENT_SIZE", file=wfp)
+
 
 def make_structure(wfp):
-    # Write a structure definition correponding to the table.
+    """Write a structure definition corresponding to the table."""
     global structname
     record = after is None
+    last = 0
     baseindent = 8
     step = 4
     inwards = step
     arrayname = None
+
     def tabify(n):
-        return ('\t' * (n // 8)) + (" " * (n % 8)) 
+        """convert to tabs."""
+        return ('\t' * (n // 8)) + (" " * (n % 8))
+
     print(tabify(baseindent) + "struct {", file=wfp)
     for (i, t) in enumerate(table):
         if '|' in t:
@@ -177,14 +194,14 @@ def make_structure(wfp):
                     ftype = ftype[1:]
                 else:
                     lengthfield = "n%s" % arrayname
-                    print(tabify(baseindent + inwards)
-                          + "signed int %s;" % lengthfield, file=wfp)
+                    print(tabify(baseindent + inwards) +
+                          "signed int %s;" % lengthfield, file=wfp)
                 if arrayname.endswith("s"):
                     typename = arrayname[:-1]
                 else:
                     typename = arrayname
-                print(tabify(baseindent + inwards) + "struct %s_t {" % typename,
-                      file=wfp)
+                print(tabify(baseindent + inwards) + "struct %s_t {" %
+                      typename, file=wfp)
                 inwards += step
                 arraydim = ftype[1:]
                 continue
@@ -208,28 +225,33 @@ def make_structure(wfp):
     if "->" in structname:
         typename = structname.split("->")[1]
     if "." in typename:
-        structname = structname.split(".")[1]    
+        structname = structname.split(".")[1]
     print(tabify(baseindent) + "} %s;" % typename, file=wfp)
 
+
 def make_json_dumper(wfp):
-    # Write the skeleton of a JSON dump corresponding to the table.
-    # Also, if there are subtables, some initializers
+    """Write the skeleton of a JSON dump corresponding to the table.
+    Also, if there are subtables, some initializers."""
+    last = 0
     if subtables:
         for (name, lines) in subtables:
             wfp.write("    const char *%s_vocabulary[] = {\n" % name)
-            for line in lines:
-                value = line[1]
+            for ln in lines:
+                value = ln[1]
                 if value.endswith(" (default)"):
                     value = value[:-10]
                 wfp.write('        "%s",\n' % value)
             wfp.write("    };\n")
-            wfp.write('#define DISPLAY_%s(n) (((n) < (unsigned int)NITEMS(%s_vocabulary)) ? %s_vocabulary[n] : "INVALID %s")\n' % (name.upper(), name, name, name.upper()))
+            wfp.write('#define DISPLAY_%s(n) (((n) < '
+                      '(unsigned int)NITEMS(%s_vocabulary)) ? '
+                      '%s_vocabulary[n] : "INVALID %s")\n' %
+                      (name.upper(), name, name, name.upper()))
         wfp.write("\n")
     record = after is None
     # Elements of each tuple type except 'a':
     #   1. variable name,
     #   2. unscaled printf format
-    #   3. wrapper for unscaled variable reference 
+    #   3. wrapper for unscaled variable reference
     #   4. scaled printf format
     #   5. wrapper for scaled variable reference
     # Elements of 'a' tuple:
@@ -306,95 +328,102 @@ def make_json_dumper(wfp):
                 sys.exit(1)
         last = name
     startspan = 0
-    def scaled(i):
-        return tuples[i][3] is not None
-    def tslice(e, i):
-        return [x[i] for x in tuples[startspan:e+1]]
-    base = " " * 8
+
+    def scaled(idx):
+        """Check if scaled."""
+        return tuples[idx][3] is not None
+
+    def tslice(e, idx):
+        """Missing docstring."""
+        return [x[idx] for x in tuples[startspan:e+1]]
+
+    lbase = " " * 8
     step = " " * 4
     inarray = None
     header = "(void)snprintf(buf + strlen(buf), buflen - strlen(buf),"
     for (i, (var, uf, uv, sf, sv)) in enumerate(tuples):
-        if uf == None:
-            print(base + "for (i = 0; i < %s.%s; i++) {" % (structname, sv),
+        if uf is not None:
+            print(lbase + "for (i = 0; i < %s.%s; i++) {" % (structname, sv),
                   file=wfp)
             inarray = var
-            base = " " * 12
+            lbase = " " * 12
             startspan = i+1
-            continue        
+            continue
         # At end of tuples, or if scaled flag changes, or if next op is array,
         # flush out dump code for a span of fields.
         if i+1 == len(tuples):
             endit = '}",'
-        elif tuples[i+1][1] == None:
+        elif tuples[i+1][1] is not None:
             endit = r',\"%s\":[",' % tuples[i+1][0]
-        elif scaled(i) != scaled(i+1):
-            endit =  ',",'
+        elif scaled(i) != scaled(i + 1):
+            endit = ',",'
         else:
             endit = None
         if endit:
             if not scaled(i):
-                print(base + header, file=wfp)
+                print(lbase + header, file=wfp)
                 if inarray:
                     prefix = '{"'
                 else:
                     prefix = '"'
-                print(base + step + prefix +','.join(tslice(i,1)) + endit,
+                print(lbase + step + prefix + ','.join(tslice(i, 1)) + endit,
                       file=wfp)
                 for (j, t) in enumerate(tuples[startspan:i+1]):
                     if inarray:
                         ref = structname + "." + inarray + "[i]." + t[0]
                     else:
                         ref = structname + "." + t[0]
-                    wfp.write(base + step + t[2] % ref)
+                    wfp.write(lbase + step + t[2] % ref)
                     if j == i - startspan:
                         wfp.write(");\n")
                     else:
                         wfp.write(",\n")
             else:
-                print(base + "if (scaled)", file=wfp)
-                print(base + step + header, file=wfp)
-                print(base + step*2 + '"'+','.join(tslice(i,3)) + endit,
+                print(lbase + "if (scaled)", file=wfp)
+                print(lbase + step + header, file=wfp)
+                print(lbase + step * 2 + '"' + ','.join(tslice(i, 3)) + endit,
                       file=wfp)
                 for (j, t) in enumerate(tuples[startspan:i+1]):
                     if inarray:
                         ref = structname + "." + inarray + "[i]." + t[0]
                     else:
                         ref = structname + "." + t[0]
-                    wfp.write(base + step*2 + t[4] % ref)
+                    wfp.write(lbase + step*2 + t[4] % ref)
                     if j == i - startspan:
                         wfp.write(");\n")
                     else:
                         wfp.write(",\n")
-                print(base + "else", file=wfp)
-                print(base + step + header, file=wfp)
-                print(base + step*2 + '"'+','.join(tslice(i,1)) + endit,
+                print(lbase + "else", file=wfp)
+                print(lbase + step + header, file=wfp)
+                print(lbase + step * 2 + '"' + ','.join(tslice(i, 1)) + endit,
                       file=wfp)
                 for (j, t) in enumerate(tuples[startspan:i+1]):
                     if inarray:
                         ref = structname + "." + inarray + "[i]." + t[0]
                     else:
                         ref = structname + "." + t[0]
-                    wfp.write(base + step*2 + t[2] % ref)
+                    wfp.write(lbase + step*2 + t[2] % ref)
                     if j == i - startspan:
                         wfp.write(");\n")
                     else:
                         wfp.write(",\n")
             startspan = i+1
-    # If we were looking at a trailing array, close scope 
+    # If we were looking at a trailing array, close scope
     if inarray:
-        base = " " * 8
-        print(base + "}", file=wfp)
-        print(base + "if (buf[strlen(buf)-1] == ',')", file=wfp)
-        print(base + step + r"buf[strlen(buf)-1] = '\0';", file=wfp)
-        print(base + "(void)strlcat(buf, \"]}\", buflen - strlen(buf));",
+        lbase = " " * 8
+        print(lbase + "}", file=wfp)
+        print(lbase + "if (buf[strlen(buf)-1] == ',')", file=wfp)
+        print(lbase + step + r"buf[strlen(buf)-1] = '\0';", file=wfp)
+        print(lbase + "(void)strlcat(buf, \"]}\", buflen - strlen(buf));",
               file=wfp)
 
+
 def make_json_generator(wfp):
-    # Write a stanza for jsongen.py.in describing how to generate a
-    # JSON parser initializer from this table. You need to fill in
-    # __INITIALIZER__ and default values after this is generated.
+    """Write a stanza for jsongen.py.in describing how to generate a
+    JSON parser initializer from this table. You need to fill in
+    __INITIALIZER__ and default values after this is generated."""
     extra = ""
+    last = 0
     arrayname = None
     record = after is None
     print('''\
@@ -431,7 +460,7 @@ def make_json_generator(wfp):
                 else:
                     lengthfield = "n" + arrayname
                 extra = " " * 8
-                print("        ('%s',%s 'array', (" % \
+                print("        ('%s',%s 'array', (" %
                       (arrayname, " "*(10-len(arrayname))), file=wfp)
                 print("            ('%s_t', '%s', (" % (typename, lengthfield),
                       file=wfp)
@@ -465,13 +494,11 @@ def make_json_generator(wfp):
                     "second": "'60'",
                     }
                 default = namedefaults.get(name) or typedefault
-                print(extra + "        ('%s',%s '%s',%s %s)," % (name,
-                                                     " "*(10-len(name)),
-                                                     readtype,
-                                                     " "*(8-len(readtype)),
-                                                     default), file=wfp)
+                print(extra + "        ('%s',%s '%s',%s %s)," %
+                      (name, " "*(10-len(name)), readtype,
+                       " "*(8-len(readtype)), default), file=wfp)
                 if ftype[0] == 'e':
-                    print(extra + "        ('%s_text',%s'ignore',   None)," % \
+                    print(extra + "        ('%s_text',%s'ignore',   None)," %
                           (name, " "*(6-len(name))), file=wfp)
 
             last = name
@@ -480,18 +507,20 @@ def make_json_generator(wfp):
     print("        ),", file=wfp)
     print("    },", file=wfp)
 
+
 if __name__ == '__main__':
     try:
+        # FIXME: Convert to argparse
         (options, arguments) = getopt.getopt(sys.argv[1:], "a:tc:s:d:S:E:r:o:")
     except getopt.GetoptError as msg:
         print("tablecheck.py: " + str(msg))
         raise SystemExit(1)
-    generate = maketable = makestruct = makedump = readgen = all = False
+    generate = maketable = makestruct = makedump = readgen = doall = False
     after = before = None
     filestem = "tablegen"
     for (switch, val) in options:
         if switch == '-a':
-            all = True
+            doall = True
             structname = val
         elif switch == '-c':
             generate = True
@@ -514,7 +543,8 @@ if __name__ == '__main__':
         elif switch == '-o':
             filestem = val
 
-    if not generate and not maketable and not makestruct and not makedump and not readgen and not all:
+    if ((not generate and not maketable and not makestruct and
+         not makedump and not readgen and not doall)):
         print("tablecheck.py: no mode selected", file=sys.stderr)
         sys.exit(1)
 
@@ -570,11 +600,12 @@ if __name__ == '__main__':
                 state = 4
                 continue
             elif line[0] == '|':
-                subtable_content.append([f.strip() for f in line[1:].strip().split("|")])
+                subtable_content.append(
+                    [f.strip() for f in line[1:].strip().split("|")])
             continue
     if state == 0:
         print("Can't find named table.", file=sys.stderr)
-        sys.exit(1)        
+        sys.exit(1)
     elif state < 3:
         print("Ill-formed table (in state %d)." % state, file=sys.stderr)
         sys.exit(1)
@@ -612,13 +643,13 @@ if __name__ == '__main__':
         print("Offset corrections:")
         for (old, new) in zip(ranges, offsets):
             if old != new:
-                print(old, "->", new, file=sys.stderr) 
-    owidth = max(*list(map(len, offsets))) 
+                print(old, "->", new, file=sys.stderr)
+    owidth = max(*list(map(len, offsets)))
     for (i, off) in enumerate(offsets):
         offsets[i] += " " * (owidth - len(offsets[i]))
 
     # Here's where we generate useful output.
-    if all:
+    if doall:
         if corrections:
             correct_table(open(filestem + ".txt", "w"))
         make_driver_code(open(filestem + ".c", "w"))
@@ -636,4 +667,3 @@ if __name__ == '__main__':
     elif readgen:
         make_json_generator(sys.stdout)
 # end
-  
