@@ -96,33 +96,8 @@ void gpsd_release_reporting_lock(void)
     }
 }
 
-// copy inbuf string to outbuf, replacingg non-printing characters
-char *visibilize(char *outbuf, size_t outlen, const char *inbuf, size_t inlen)
-{
-    const char *sp;
-    size_t next = 0;
-
-    outbuf[0] = '\0';
-    // FIXME!! snprintf() when strlcat() will do!
-    for (sp = inbuf; sp < inbuf + inlen && (next + 6) < outlen; sp++) {
-        int ret;
-        if (isprint((unsigned char)*sp)) {
-            ret = snprintf(&outbuf[next], 2, "%c", *sp);
-        } else {
-            ret = snprintf(&outbuf[next], 6, "\\x%02x",
-                           0x00ff & (unsigned)*sp);
-        }
-        if (1 > ret) {
-            // error, or ran out of space
-            break;
-        }
-        next += ret;
-    }
-    return outbuf;
-}
-
-
 // assemble msg in vprintf(3) style, use errout hook or syslog for delivery
+// FIXME: duplicated in gpsd/libgpsd_core.c
 static void gpsd_vlog(const int errlevel,
                       const struct gpsd_errout_t *errout,
                       char *outbuf, size_t outlen,
@@ -203,7 +178,7 @@ static void gpsd_vlog(const int errlevel,
     vsnprintf(outbuf, outlen, buf, ap);
 
     // this was carzy expensive, just fix the bad log calls
-    // visibilize(outbuf, outlen, buf, strlen(buf));
+    // gps_visibilize(outbuf, outlen, buf, strlen(buf));
 
     if (getpid() == getsid(getpid())) {
         // I think this calls syslog() only when daemonized
@@ -243,7 +218,7 @@ void gpsd_log(const int errlevel, const struct gpsd_errout_t *errout,
 const char *gpsd_prettydump(struct gps_device_t *session)
 {
     return gpsd_packetdump(session->msgbuf, sizeof(session->msgbuf),
-                           (char *)session->lexer.outbuffer,
+                           session->lexer.outbuffer,
                            session->lexer.outbuflen);
 }
 
@@ -330,7 +305,7 @@ void gps_context_init(struct gps_context_t *context,
     context->serial_write = gpsd_serial_write;
 
     errout_reset(&context->errout);
-    context->errout.label = (char *)label;
+    context->errout.label = label;
 
     (void)pthread_mutex_init(&report_mutex, NULL);
 }
@@ -477,7 +452,7 @@ void gpsd_clear(struct gps_device_t *session)
     // clear the private data union
     memset((void *)&session->driver, '\0', sizeof(session->driver));
     // set up the context structure for the PPS thread monitor
-    memset((void *)&session->pps_thread, 0, sizeof(session->pps_thread));
+    memset_volatile(&session->pps_thread, 0, sizeof(session->pps_thread));
     session->pps_thread.devicefd = session->gpsdata.gps_fd;
     session->pps_thread.devicename = session->gpsdata.dev.path;
     session->pps_thread.log_hook = ppsthread_log;
@@ -553,7 +528,7 @@ int parse_uri_dest(char *s, char **host, char **service, char **device)
             *s = '\0';
         }
         // nothing in service besides / --> trigger use of default port
-        if (0 == strlen(*service)) {
+        if ('\0' == *service[0]) {
             *service = NULL;
         }
     }
@@ -900,6 +875,7 @@ static gps_mask_t fill_dop(const struct gpsd_errout_t *errout,
     memset(satpos, 0, sizeof(satpos));
 
     for (n = k = 0; k < gpsdata->satellites_visible; k++) {
+        // This double counts single sats where we got 2 signals from them.
         if (!gpsdata->skyview[k].used) {
              // skip unused sats
              continue;
@@ -927,7 +903,7 @@ static gps_mask_t fill_dop(const struct gpsd_errout_t *errout,
         satpos[n][2] = sin(sp->elevation * DEG_2_RAD);
         satpos[n][3] = 1;
         GPSD_LOG(LOG_INF, errout,
-                 "CORE: PRN%3d az %5.1f el %4.1f (%9.6f, %9.6f, %9.6f)\n",
+                 "CORE: PRN %3d az %5.1f el %4.1f (%9.6f, %9.6f, %9.6f)\n",
                  gpsdata->skyview[k].PRN,
                  gpsdata->skyview[k].azimuth,
                  gpsdata->skyview[k].elevation,
@@ -1641,9 +1617,10 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
         newlen = session->device_type->get_packet(session);
         // coverity[deref_ptr]
         GPSD_LOG(LOG_RAW, &session->context->errout,
-                 "CORE: %s is known to be %s\n",
+                 "CORE: %s is known to be %s, packet type %d\n",
                  session->gpsdata.dev.path,
-                 session->device_type->type_name);
+                 session->device_type->type_name,
+                 session->lexer.type);
     } else {
         newlen = generic_get(session);
     }
@@ -1840,13 +1817,13 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
              gpsd_prettydump(session));
 
     // Get data from current packet into the fix structure
-    if (COMMENT_PACKET != session->lexer.type) {
-        if (NULL != session->device_type &&
-            NULL != session->device_type->parse_packet) {
+    if (COMMENT_PACKET != session->lexer.type &&
+        BAD_PACKET != session->lexer.type &&
+        NULL != session->device_type &&
+        NULL != session->device_type->parse_packet) {
             received |= session->device_type->parse_packet(session);
             GPSD_LOG(LOG_SPIN, &session->context->errout,
                      "CORE: parse_packet() = %s\n", gps_maskdump(received));
-        }
     }
 
     /*
@@ -2168,6 +2145,10 @@ void gpsd_wrap(struct gps_device_t *session)
     }
 }
 
+/* gpsd_zero_satellites(), initialize the skyview (satellite_t)
+ *
+ * return; void
+ */
 void gpsd_zero_satellites( struct gps_data_t *out)
 {
     int sat;
@@ -2180,7 +2161,9 @@ void gpsd_zero_satellites( struct gps_data_t *out)
         out->skyview[sat].azimuth = NAN;
         out->skyview[sat].elevation = NAN;
         out->skyview[sat].ss = NAN;
+        out->skyview[sat].prRes = NAN;
         out->skyview[sat].freqid = -1;
+        out->skyview[sat].qualityInd = -1;
     }
 #if 0
     /*
