@@ -94,7 +94,7 @@ static char *ntrip_field_iterate(char *start,
         if (!prev) {
             return NULL;
         }
-        s = prev + strlen(prev) + 1;
+        s = prev + strnlen(prev, BUFSIZ) + 1;
         if (s >= eol) {
             return NULL;
         }
@@ -180,8 +180,9 @@ static void ntrip_str_parse(char *str, size_t len,
     s = ntrip_field_iterate(NULL, s, eol, errout);
     // <compr-encryp>
     if (NULL != (s = ntrip_field_iterate(NULL, s, eol, errout))) {
-        if ((0 == strcmp(" ", s)) ||
-            (0 == strlen(s)) ||
+
+        if (('\0' == s[0]) ||
+            (0 == strcmp(" ", s)) ||
             (0 == strcasecmp("none", s))) {
             hold->compr_encryp = CMP_ENC_NONE;
         } else {
@@ -324,8 +325,8 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
 
             if (str_starts_with(line, NTRIP_STR)) {
                 // parse STR
-                ntrip_str_parse(line + strlen(NTRIP_STR),
-                                (size_t) (llen - strlen(NTRIP_STR)),
+                ntrip_str_parse(line + sizeof(NTRIP_STR) - 1,
+                                (size_t)(llen - (sizeof(NTRIP_STR) - 1)),
                                 &hold, &device->context->errout);
 
                 if (0 == strcmp(device->ntrip.stream.mountpoint,
@@ -385,7 +386,7 @@ static int ntrip_sourcetable_parse(struct gps_device_t *device)
             }
             // else ???
 
-            llen += strlen(NTRIP_BR);
+            llen += sizeof(NTRIP_BR) - 1;
             line += llen;        // point to start of next line
             len -= llen;         // calculate remaining data in buf.
             GPSD_LOG(LOG_IO, &device->context->errout,
@@ -417,7 +418,7 @@ static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
                                   struct gpsd_errout_t *errout)
 {
     int dsock;
-    ssize_t r;
+    ssize_t r, blen;
     char buf[BUFSIZ];
     char outbuf[BUFSIZ];
 
@@ -429,26 +430,31 @@ static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
                  stream->url, netlib_errstr(dsock), dsock);
         return -1;
     }
-    (void)snprintf(buf, sizeof(buf),
-            "GET / HTTP/1.1\r\n"
-            "Ntrip-Version: Ntrip/2.0\r\n"
-            "User-Agent: NTRIP gpsd/%s\r\n"
-            "Host: %s\r\n"
-            "Connection: close\r\n"
-            "\r\n", VERSION, stream->host);
+    blen = snprintf(buf, sizeof(buf),
+                    "GET / HTTP/1.1\r\n"
+                    "Ntrip-Version: Ntrip/2.0\r\n"
+                    "User-Agent: NTRIP gpsd/%s\r\n"
+                    "Host: %s\r\n"
+                    "Connection: close\r\n"
+                    "\r\n", VERSION, stream->host);
+    if (1 > blen) {
+        GPSD_LOG(LOG_ERROR, errout,
+                 "NTRIP: ntrip_stream_req_probe(%s) snprintf() fail\n",
+                 stream->url);
+        return -1;
+    }
 
     GPSD_LOG(LOG_IO, errout,
              "NTRIP: ntrip_stream_req_probe(%s) fd %d sending >%s<\n",
              stream->url, dsock,
-             visibilize(outbuf, sizeof(outbuf), buf,
-                        strnlen(buf, sizeof(buf))));
+             gps_visibilize(outbuf, sizeof(outbuf), buf, blen));
 
-    r = write(dsock, buf, strlen(buf));
-    if ((ssize_t)strlen(buf) != r) {
+    r = write(dsock, buf, blen);
+    if (blen != r) {
         GPSD_LOG(LOG_ERROR, errout,
-                 "NTRIP: stream write error %s on fd %d "
+                 "NTRIP: stream write error %s(%d) on fd %d "
                  "during probe request %zd\n",
-                 strerror(errno), dsock, r);
+                 strerror(errno), errno, dsock, r);
         (void)close(dsock);
         return -1;
     }
@@ -461,15 +467,12 @@ static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream,
  * Return: 0 == OK
  *         -1 = auth error
  */
-static int ntrip_auth_encode(const struct ntrip_stream_t *stream,
-                             const char *auth,
-                             char buf[],
-                             size_t size)
+static int ntrip_auth_encode(struct ntrip_stream_t *stream)
 {
     char authenc[64];       // base 64 encoding of auth (username:password)
     int ret = 0;
 
-    memset(buf, 0, size);
+    memset(stream->authStr, 0, sizeof(stream->authStr));
     switch (stream->authentication) {
     case AUTH_NONE:
         // nothing to do.
@@ -477,17 +480,16 @@ static int ntrip_auth_encode(const struct ntrip_stream_t *stream,
     case AUTH_BASIC:
         // RFC 7617 Basic Access Authentication.
         // username may not contain a colon (")
-        if (NULL == auth) {
-            ret = -1;
-            break;
-        }
         memset(authenc, 0, sizeof(authenc));
-        if (0 > b64_ntop((unsigned char *)auth, strlen(auth), authenc,
-                         sizeof(authenc) - 1)) {
+        if (0 > b64_ntop((const unsigned char *)stream->credentials,
+                         strnlen(stream->credentials,
+                                 sizeof(stream->credentials)),
+                         authenc, sizeof(authenc) - 1)) {
             ret = -1;
             break;
         }
-        (void)snprintf(buf, size - 1, "Authorization: Basic %s\r\n", authenc);
+        (void)snprintf(stream->authStr, sizeof(stream->authStr),
+                       "Authorization: Basic %s\r\n", authenc);
         break;
     case AUTH_DIGEST:
         // TODO: support digest authentication, who needs it?
@@ -521,6 +523,7 @@ static socket_t ntrip_stream_get_req(const struct ntrip_stream_t *stream,
     int dsock;
     char buf[BUFSIZ];
     char outbuf[BUFSIZ];
+    ssize_t cnt, cnt1;
 
     // open blocking
     dsock = netlib_connectsock(AF_UNSPEC, stream->host, stream->port, "tcp");
@@ -535,23 +538,28 @@ static socket_t ntrip_stream_get_req(const struct ntrip_stream_t *stream,
              "NTRIP: netlib_connectsock() returns socket on fd %d\n",
              dsock);
 
-    (void)snprintf(buf, sizeof(buf),
-            "GET /%s HTTP/1.1\r\n"
-            "Ntrip-Version: Ntrip/2.0\r\n"
-            "User-Agent: NTRIP gpsd/%s\r\n"
-            "Host: %s\r\n"
-            "Accept: rtk/rtcm, dgps/rtcm\r\n"
-            "%s"
-            "Connection: close\r\n"
-            "\r\n", stream->mountpoint, VERSION, stream->host,
-            stream->authStr);
+    cnt = snprintf(buf, sizeof(buf),
+                   "GET /%s HTTP/1.1\r\n"
+                   "Ntrip-Version: Ntrip/2.0\r\n"
+                   "User-Agent: NTRIP gpsd/%s\r\n"
+                   "Host: %s\r\n"
+                   "Accept: rtk/rtcm, dgps/rtcm\r\n"
+                   "%s"
+                   "Connection: close\r\n"
+                   "\r\n", stream->mountpoint, VERSION, stream->host,
+                   stream->authStr);
+    if (1 > cnt) {
+        GPSD_LOG(LOG_ERROR, errout,
+                 "NTRIP: netlib_connectsock() snprintf fail<\n");
+        return -1;
+    }
 
     GPSD_LOG(LOG_IO, errout,
              "NTRIP: netlib_connectsock() sending >%s<\n",
-             visibilize(outbuf, sizeof(outbuf), buf,
-                        strnlen(buf, sizeof(buf))));
+             gps_visibilize(outbuf, sizeof(outbuf), buf, cnt));
 
-    if ((ssize_t)strlen(buf) != write(dsock, buf, strlen(buf))) {
+    cnt1 = write(dsock, buf, cnt);
+    if (cnt != cnt1) {
         GPSD_LOG(LOG_ERROR, errout,
                  "NTRIP: stream write error %s(%d) on fd %d during "
                  "get request\n",
@@ -636,7 +644,6 @@ int ntrip_parse_url(const struct gpsd_errout_t *errout,
     char *host = NULL;                   // hostname, IPv4 or IPv6
     char *port = NULL;
     char *mountpoint = NULL;             // mount point
-    size_t len = 0;
 
     // save original URL
     strlcpy(stream->url, fullurl, sizeof(stream->url) - 1);
@@ -654,8 +661,7 @@ int ntrip_parse_url(const struct gpsd_errout_t *errout,
     mountpoint = slash + 1;
     // dup now ends in host or host:port
 
-    len = strlen(mountpoint);
-    if (0 == len) {
+    if ('\0' == mountpoint[0]) {
         // this also handle the trailing / case
         GPSD_LOG(LOG_ERROR, errout,
                  "NTRIP: ntrip_parse_url(%s) missing mountpoint.\n", fullurl);
@@ -842,6 +848,7 @@ int ntrip_open(struct gps_device_t *device, char *orig)
     socket_t ret = -1;
     char buf[BUFSIZ];
     char outbuf[BUFSIZ];
+    ssize_t blen;
 
     GPSD_LOG(LOG_PROG, &device->context->errout,
              "NTRIP: ntrip_open(%s) fd %d state = %d\n",
@@ -913,10 +920,7 @@ int ntrip_open(struct gps_device_t *device, char *orig)
                  device->ntrip.stream.authentication,
                  device->ntrip.stream.fee,
                  device->ntrip.stream.bitrate);
-        if (0 != ntrip_auth_encode(&device->ntrip.stream,
-                                   device->ntrip.stream.credentials,
-                                   device->ntrip.stream.authStr,
-                                   sizeof(device->ntrip.stream.authStr))) {
+        if (0 != ntrip_auth_encode(&device->ntrip.stream)) {
             device->ntrip.conn_state = NTRIP_CONN_ERR;
             return -1;
         }
@@ -955,24 +959,29 @@ int ntrip_open(struct gps_device_t *device, char *orig)
         // Need to send GET within about 40 seconds or caster tiems out.
         // FIXME: partially duplicates  ntrip_stream_get_req()
         // try a write, it will fail if connection still in process, or failed.
-        (void)snprintf(buf, sizeof(buf),
-                "GET /%s HTTP/1.1\r\n"
-                "Ntrip-Version: Ntrip/2.0\r\n"
-                "User-Agent: NTRIP gpsd/%s\r\n"
-                "Host: %s\r\n"
-                "Accept: rtk/rtcm, dgps/rtcm\r\n"
-                "%s"
-                "Connection: close\r\n"
-                "\r\n", device->ntrip.stream.mountpoint, VERSION,
-                device->ntrip.stream.host,
-                device->ntrip.stream.authStr);
+        blen = snprintf(buf, sizeof(buf),
+                        "GET /%s HTTP/1.1\r\n"
+                        "Ntrip-Version: Ntrip/2.0\r\n"
+                        "User-Agent: NTRIP gpsd/%s\r\n"
+                        "Host: %s\r\n"
+                        "Accept: rtk/rtcm, dgps/rtcm\r\n"
+                        "%s"
+                        "Connection: close\r\n"
+                        "\r\n", device->ntrip.stream.mountpoint, VERSION,
+                        device->ntrip.stream.host,
+                        device->ntrip.stream.authStr);
+
+        if (1 > blen) {
+            GPSD_LOG(LOG_ERROR, &device->context->errout,
+                     "NTRIP: ntrip_open() snprintf fail<\n");
+            return -1;
+        }
 
         GPSD_LOG(LOG_IO, &device->context->errout,
                  "NTRIP: ntrip_open() sending >%s<\n",
-                  visibilize(outbuf, sizeof(outbuf), buf, strlen(buf)));
+                  gps_visibilize(outbuf, sizeof(outbuf), buf, blen));
 
-        if ((ssize_t)strlen(buf) != write(device->gpsdata.gps_fd, buf,
-                                          strlen(buf))) {
+        if (blen != write(device->gpsdata.gps_fd, buf, blen)) {
             GPSD_LOG(LOG_ERROR, &device->context->errout,
                      "NTRIP: stream write error %s(%d) on fd %d during "
                      "get request\n",
@@ -1015,11 +1024,12 @@ void ntrip_report(struct gps_context_t *context,
         0 == (count % 5)) {
         if (-1 < caster->gpsdata.gps_fd) {
             char buf[BUFSIZ];
-            ssize_t ret;
+            ssize_t ret, blen;
 
             gpsd_position_fix_dump(gps, buf, sizeof(buf));
-            ret = write(caster->gpsdata.gps_fd, buf, strlen(buf));
-            if ((ssize_t)strlen(buf) == ret) {
+            blen = strnlen(buf, sizeof(buf));
+            ret = write(caster->gpsdata.gps_fd, buf, blen);
+            if (blen == ret) {
                 GPSD_LOG(LOG_IO, &context->errout, "NTRIP: => caster %s\n",
                          buf);
             } else if (0 > ret) {
