@@ -201,6 +201,13 @@ double safe_atof(const char *string)
         }
         while (isdigit((int) *p)) {
             exp = exp * 10 + (*p - '0');
+            if (1024 < exp) {
+                if (true == expSign) {
+                    // exponent underflow!
+                    return 0.0;
+                } // else  exponent overflow!
+                return INFINITY;
+            }
             p += 1;
         }
     }
@@ -308,6 +315,7 @@ void gps_clear_fix(struct gps_fix_t *fixp)
     fixp->wanglet = NAN;
     fixp->wspeedr = NAN;
     fixp->wspeedt = NAN;
+    fixp->wtemp = NAN;
     gps_clear_base(&fixp->base);
 }
 
@@ -326,12 +334,14 @@ void gps_clear_att(struct attitude_t *attp)
     attp->gyro_y = NAN;
     attp->gyro_z = NAN;
     attp->heading = NAN;
+    attp->mheading = NAN;
     attp->mag_len = NAN;
     attp->mag_x = NAN;
     attp->mag_y = NAN;
     attp->mag_z = NAN;
     attp->pitch = NAN;
     attp->roll = NAN;
+    attp->rot = NAN;
     attp->temp = NAN;
     attp->yaw = NAN;
     gps_clear_base(&attp->base);
@@ -510,6 +520,9 @@ void gps_merge_fix(struct gps_fix_t *to,
             to->wspeedt = from->wspeedt;
         }
     }
+    if (0 != isfinite(from->wtemp)) {
+        to->wtemp = from->wtemp;
+    }
 }
 
 /* mkgmtime(tm)
