@@ -39,15 +39,6 @@
 #ifdef SOCKET_EXPORT_ENABLE
 #include "../include/gps_json.h"
 
-struct privdata_t
-{
-    bool newstyle;
-    // data buffered from the last read
-    ssize_t waiting;
-    char buffer[GPS_JSON_RESPONSE_MAX * 2];
-    int waitcount;
-};
-
 #ifdef HAVE_WINSOCK2_H
 static bool need_init = TRUE;
 static bool need_finish = TRUE;
@@ -129,15 +120,11 @@ int gps_sock_open(const char *host, const char *port,
 #endif  // USE_QT
 
     // set up for line-buffered I/O over the daemon socket
-    gpsdata->privdata = (void *)malloc(sizeof(struct privdata_t));
+    gpsdata->privdata =
+        (struct privdata_t *)calloc(1, sizeof(struct privdata_t));
     if (NULL == gpsdata->privdata) {
         return -1;
     }
-    PRIVATE(gpsdata)->newstyle = false;
-    PRIVATE(gpsdata)->waiting = 0;
-    PRIVATE(gpsdata)->buffer[0] = 0;
-
-    PRIVATE(gpsdata)->waitcount = 0;
     return 0;
 }
 
@@ -394,9 +381,12 @@ int gps_sock_read(struct gps_data_t *gpsdata, char *message, int message_len)
  * gps_unpack() currently returns 0 in all cases, but should it ever need to
  * return an error status, it must be < 0.
  */
-int gps_unpack(char *buf, struct gps_data_t *gpsdata)
+int gps_unpack(const char *buf, struct gps_data_t *gpsdata)
 {
-    libgps_debug_trace((DEBUG_CALLS, "gps_unpack(%s)\n", buf));
+    char vbuf[GPS_JSON_COMMAND_MAX];
+    libgps_debug_trace((DEBUG_CALLS, "gps_unpack(%s)\n",
+                        gps_visibilize(vbuf, sizeof(vbuf),
+                                       buf, strnlen(buf, sizeof(vbuf)))));
 
     // detect and process a JSON response
     if ('{' == buf[0]) {
@@ -407,7 +397,8 @@ int gps_unpack(char *buf, struct gps_data_t *gpsdata)
                '\0' != next[0][0]) {
             libgps_debug_trace((DEBUG_CALLS,
                                "gps_unpack() segment parse '%s'\n",
-                               *next));
+                               gps_visibilize(vbuf, sizeof(vbuf), *next,
+                                              strnlen(*next, sizeof(vbuf)))));
             if (-1 == libgps_json_unpack(*next, gpsdata, next)) {
                 break;
             }
@@ -438,11 +429,14 @@ const char *gps_sock_data(const struct gps_data_t *gpsdata)
  * Return: 0 -- success
  * Return: negative -- fail
  */
+// FIXME: pass in buf_len
 int gps_sock_send(struct gps_data_t *gpsdata, const char *buf)
 {
+    size_t buf_len = strnlen(buf, BUFSIZ);
+
 #ifdef USE_QT
     QTcpSocket *sock = (QTcpSocket *) gpsdata->gps_fd;
-    sock->write(buf, strlen(buf));
+    sock->write(buf, buf_len);
     if (sock->waitForBytesWritten()) {
         return 0;
     }
@@ -451,21 +445,22 @@ int gps_sock_send(struct gps_data_t *gpsdata, const char *buf)
 #else   // USE_QT
     ssize_t sent;
 #ifdef HAVE_WINSOCK2_H
-    sent = send(gpsdata->gps_fd, buf, strlen(buf), 0);
+    sent = send(gpsdata->gps_fd, buf, buf_len, 0);
 #else
-    sent = write(gpsdata->gps_fd, buf, strlen(buf));
+    sent = write(gpsdata->gps_fd, buf, buf_len);
 #endif /* HAVE_WINSOCK2_H */
-    if ((ssize_t)strlen(buf) == sent) {
+    if ((ssize_t)buf_len == sent) {
         return 0;
     }
     (void)fprintf(stderr, "gps_sock_send() write %ld, s/b %ld\n",
-                  (long)sent, (long)strlen(buf));
+                  (long)sent, (long)buf_len);
 #endif  // USE_QT
     return -1;
 }
 
 // ask gpsd to stream reports at you, hiding the command details
-int gps_sock_stream(struct gps_data_t *gpsdata, unsigned int flags, void *d)
+int gps_sock_stream(struct gps_data_t *gpsdata, watch_t flags,
+                    const char *d)
 {
     char buf[GPS_JSON_COMMAND_MAX] = "?WATCH={\"enable\":";
 
@@ -526,7 +521,7 @@ int gps_sock_stream(struct gps_data_t *gpsdata, unsigned int flags, void *d)
             (void)strlcat(buf, ",\"pps\":true", sizeof(buf));
         }
         if (flags & WATCH_DEVICE) {
-            str_appendf(buf, sizeof(buf), ",\"device\":\"%s\"", (char *)d);
+            str_appendf(buf, sizeof(buf), ",\"device\":\"%s\"", d);
         }
     }
     (void)strlcat(buf, "};", sizeof(buf));
@@ -552,12 +547,10 @@ int gps_sock_mainloop(struct gps_data_t *gpsdata, int timeout,
         }
         status = gps_read(gpsdata, NULL, 0);
 
-        if (-1 == status) {
+        if (0 > status) {
             break;
         }
-        if (0 < status) {
-            (*hook)(gpsdata);
-        }
+        (*hook)(gpsdata);
     }
     return -2;
 }
